// marked to delete
class LSLinks {
    static showMenuIcons() {
        const leadingIcons = document.querySelectorAll('.with-leading-icon');
        const trailingIcons = document.querySelectorAll('.with-trailing-icon');
        const iconOnlyElements = document.querySelectorAll('.with-icon-only');

        leadingIcons.forEach(element => {
            const icon = element.getAttribute('data-icon');
            const isLastElement = element.getAttribute('data-last-element') || false;
            const iconHTML = `<i class="${icon}${isLastElement ? '' : ' mr-1'}"></i>`;
            element.insertAdjacentHTML('afterbegin', iconHTML);
        });

        trailingIcons.forEach(element => {
            const icon = element.getAttribute('data-icon');
            const isLastElement = element.getAttribute('data-last-element') || false;
            const iconHTML = `<i class="${icon}${isLastElement ? '' : ' mr-1'}"></i>`;
            element.insertAdjacentHTML('beforeend', iconHTML);
        });

        iconOnlyElements.forEach(element => {
            const icon = element.getAttribute('data-icon');
            const isLastElement = element.getAttribute('data-last-element') || false;
            const iconHTML = `<i class="${icon}${isLastElement ? '' : ' mr-1'}"></i>`;
            element.insertAdjacentHTML('afterbegin', iconHTML);
        });
    }
}


class LSNavigationController {

    timeout = 500;
    toggler = '[data-role="menu-toggler"]';
    fadeClass = 'fade';
    showClass = 'show';

    constructor() {
        if(LSNavigationController.instance) {
            return LSNavigationController.instance;
        }

        LSNavigationController.instance = this;
    }

    #displayFade(element) {
        if(element.classList.contains(this.showClass)) {
            element.classList.remove(this.fadeClass);
            setTimeout(() => {
                element.classList.remove(this.showClass);
            }, this.timeout);
        } else {
            element.classList.add(this.showClass);
            setTimeout(() => {
                element.classList.add(this.fadeClass);
            }, 50);
        }
    }

    #clickOutsideHandler(toggler, controlledMenu) {
        document.addEventListener('click', (event) => {
            if(!controlledMenu.contains(event.target) && !toggler.contains(event.target)) {
                this.#displayFade(controlledMenu);
                toggler.setAttribute('aria-expanded', false);
            }
        })
    }

    init(toggler = this.toggler, timeout = this.timeout, fadeClass = this.fadeClass, showClass = this.showClass) {
        this.toggler = toggler;
        this.timeout = timeout;
        this.fadeClass = fadeClass;
        this.showClass = showClass;

        this.menuObject = document.querySelector('.nav-menu');

        document.querySelectorAll(toggler).forEach(item => {
            item.addEventListener('click', () => {
                const controlledId = item.getAttribute('aria-controls');
                if(controlledId) {
                    const controlledElement = document.getElementById(controlledId);
                    if(controlledElement) {
                        this.#displayFade(controlledElement);
                        this.#clickOutsideHandler(item, controlledElement);
                    } else {
                        console.error(`Element with id ${controlledId} not found`);
                    }
                }
            });
        });
    }
}


class LSTogglers {
    
    constructor() {
        this.buttons = document.querySelectorAll('[data-toggle]:not([data-toggle=""])');
        this.buttons.forEach(button => {
            button.addEventListener('click', this.toggle.bind(this));
        });
    }

    toggle(e) {
        const target = e.target;
        const type = target.getAttribute('data-toggle');
        const targetSelector = target.getAttribute('data-controls');
        const overlay = document.createElement('div');
        overlay.id = 'offcanvas-overlay';
        overlay.className = 'offcanvas-overlay';
        const closeButtons = document.querySelectorAll('.button.close');
        let showingClass = '';

        switch(type) {
            case 'offcanvas':
                const targetElement = document.querySelector(targetSelector);
                if (targetElement.classList.contains('right-side')) {
                    targetElement.classList.add('show-right');
                    showingClass = 'show-right';
                } else {
                    targetElement.classList.add('show-left');
                    showingClass = 'show-left';
                }
                document.body.appendChild(overlay);
                overlay.style.display = 'block';
                overlay.addEventListener('click', () => {
                    targetElement.classList.remove(showingClass);
                    overlay.remove();
                });
                closeButtons.forEach(button => {
                    button.addEventListener('click', () => {
                        targetElement.classList.remove(showingClass);
                        overlay.remove();
                    });
                });
                break;
            default:
                target.classList.toggle('show');
        }
    }
}


class LSScrollPage {
    
    constructor(state = false) {
        this.sections = document.querySelectorAll('section');
        this.currentPosition = 0;
        this.isAnimating = false;
        this.init(state);
    }

    init(state = false) {
        if (state) {
            console.log('sections in init: ' + this.sections.length);
            this.scrollTo(0);
            this.createIndicator();
            this.addEvents();
        }
    }

    scrollTo(index) {
        if (index >= 0 && index < this.sections.length && !this.isAnimating) {
            this.isAnimating = true;
            const targetPosition = this.sections[index].offsetTop;
            window.scrollTo({
                top: targetPosition,
                behavior: 'smooth'
            });
            setTimeout(() => {
                this.isAnimating = false;
                this.updateIndicator();
            }, 700);
            this.currentPosition = index;
        }
    }

    createIndicator() {
        const indicator = document.querySelector('.scroll-indicator');
        if (!indicator) return;

        this.sections.forEach((section, index) => {
            const indicatorItem = document.createElement('div');
            indicator.appendChild(indicatorItem);
        });
        this.updateIndicator();
    }

    updateIndicator() {
        const indicators = document.querySelectorAll('#scroll-indicator > div');
        indicators.forEach((indicator, index) => {
            if (index === this.currentPosition) {
                indicator.classList.add('active');
            } else {
                indicator.classList.remove('active');
            }
        });
    }

    addEvents() {
        document.addEventListener('wheel', (event) => {
            event.preventDefault();
            if (this.isAnimating) return;

            if (event.deltaY > 0) {
                this.scrollTo(this.currentPosition + 1);
            } else {
                this.scrollTo(this.currentPosition - 1);
            }
        }, { passive: false });

        document.addEventListener('keydown', (event) => {
            if (this.isAnimating) return;

            switch (event.key) {
                case 'ArrowUp':
                    this.scrollTo(this.currentPosition - 1);
                    break;
                case 'ArrowDown':
                case ' ': // space
                    this.scrollTo(this.currentPosition + 1);
                    break;
            }
        });
    }
}

class LSDebug {
    static init(state = false) {
        if(state) {
            this.showWindowSize();
            this.addEvents();
        }
    }

    static showWindowSize() {
        const debugElement = document.createElement('div');
        debugElement.className = 'ls-debug';
        debugElement.innerHTML = `
            <div class="ls-debug-header">
                <span class="ls-debug-title">Window Size: ${window.innerWidth}px x ${window.innerHeight}px</span>
                <button class="ls-debug-close">&times;</button>
            </div>
        `;

        debugElement.style.position = 'fixed';
        debugElement.style.bottom = '1rem';
        debugElement.style.right = '1rem';
        debugElement.style.zIndex = '1000';
        debugElement.style.padding = '1rem';
        debugElement.style.backgroundColor = 'rgba(0, 0, 0, 0.8)';
        debugElement.style.color = '#fff';
        debugElement.style.fontFamily = 'monospace';
        debugElement.style.fontSize = '1.2rem';
        debugElement.style.borderRadius = '0.5rem';
        debugElement.style.cursor = 'move';
        document.body.appendChild(debugElement);

        const closeButton = debugElement.querySelector('.ls-debug-close');
        closeButton.addEventListener('click', () => {
            debugElement.remove();
        });

        this.makeDraggable(debugElement);
    }

    static updateWindowSize() {
        const debugElement = document.querySelector('.ls-debug .ls-debug-title');
        if(debugElement) {
            debugElement.innerText = `Window Size: ${window.innerWidth}px x ${window.innerHeight}px`;
        }
    }

    static makeDraggable(element) {
        let isDragging = false;
        let offsetX, offsetY;

        element.addEventListener('mousedown', (e) => {
            isDragging = true;
            offsetX = e.clientX - element.getBoundingClientRect().left;
            offsetY = e.clientY - element.getBoundingClientRect().top;
            element.style.transition = 'none';
        });

        document.addEventListener('mousemove', (e) => {
            if(isDragging) return;
            element.style.left = `${e.clientX - offsetX}px`;
            element.style.top = `${e.clientY - offsetY}px`;
        });

        element.addEventListener('mouseup', () => {
            isDragging = false;
            element.style.transition = '';
        });
    }
}


class LSComputeSize {
    static init(state = false) {
        if (state) {
            const elements = document.querySelectorAll('[role="compute"]');
            this.parse(elements);
        }
    }

    static parse(elements) {
        elements.forEach((element) => {
            const sizes = element.dataset.sizes;
            if (!sizes) return;

            const dimensions = sizes.split(' ');

            dimensions.forEach((dimension) => {
                const match = dimension.match(/(w|h)-(\d+)-(\w+)/);
                if (match) {
                    const [_, type, size, unit] = match;

                    if (type === 'w') {
                        element.style.width = `${size}${unit}`;
                    } else if (type === 'h') {
                        element.style.height = `${size}${unit}`;
                    }

                    LSAnimations.fadeIn(element, 1000);
                }
            });

            const lastDimension = dimensions.at(-1);
        });
    }
}

class LSFlexCards {
    static init(state = false) {
        if(state) {
            this.addEvents();
        }
    }

    static addEvents() {
        const items = document.querySelectorAll('.flexcards > *');

        items.forEach((item) => {
            item.addEventListener('click', () => {
                items.forEach((item) => item.removeAttribute('data-active'));
                item.setAttribute('data-active', 'true');
            });
        });
    }
}


class LSHorizontalTab {
    
    constructor(state = false) {
        if (state) {
            this.init();
        }
    }

    init() {
        this.initAttr = document.querySelector('.horizontal-tabs ul[role="tabs"] a[aria-selected="true"]').getAttribute('aria-controls');
        const content = document.querySelector(`div[role="horizontal-tabs"] div[role="tab-content"][aria-labelledby="${this.initAttr}"]`);
        if (content) {
            content.style.display = 'block';
        }
        this.addEvents();
    }

    addEvents() {
        const tabs = document.querySelectorAll('.horizontal-tabs ul[role="tabs"] a');
        tabs.forEach(tab => {
            tab.addEventListener('click', (event) => {
                event.preventDefault();
                
                const currentSelected = document.querySelector('.horizontal-tabs ul[role="tabs"] a[aria-selected="true"]');
                if (currentSelected) {
                    currentSelected.setAttribute('aria-selected', 'false');
                }

                tab.setAttribute('aria-selected', 'true');
                const clickedAttr = tab.getAttribute('aria-controls');

                const allContent = document.querySelectorAll('.horizontal-tabs div[role="tab-content"]');
                allContent.forEach(content => {
                    content.style.display = 'none';
                });

                const showContent = document.querySelector(`.horizontal-tabs div[role="tab-content"][aria-labelledby="${clickedAttr}"]`);
                if (showContent) {
                    showContent.style.display = 'block';
                }
            });
        });
    }
}


class LSThemeController {

    #palette = 'palette-default';
    #isDark = false;

    constructor(palette = this.#palette) {
        console.log('LSThemeController::constructor(), palette=' + palette);
        if(LSThemeController.instance) {
            console.log('LSThemeController::constructor() got instance.');
            this.use(palette);            
            return LSThemeController.instance;
        }

        this.#init(palette);
        LSThemeController.instance = this;
    }

    #init(palette) {
        console.log('LSThemeController::#init(palette) with palette=' + palette);
        this.#palette = palette;
        document.documentElement.classList.add(this.#palette);
    }

    use(newPalette) {
        console.log('LSThemeController::use(newPalette) with newPalette=' + newPalette);
        document.documentElement.classList.remove(this.#palette);
        console.log('LSThemeController::use(newPalette) removing ' + this.#palette);
        this.#palette = newPalette;
        document.documentElement.classList.add(this.#palette);
        this.#isDark = this.#palette.includes('dark');
        console.log('LSThemeController::use(newPalette) isDark=' + this.#isDark);
    }

    toggle() {
        //console.log('LSThemeController::toggle()');
        let newPalette = '';
        if(this.#isDark) {
            newPalette = this.#palette.replace('-dark', '');
        } else {
            newPalette = this.#palette + '-dark';
        }
        //console.log('LSThemeController::toggle() new classname is ' + newPalette);

        this.use(newPalette);
        this.#palette = newPalette;
        //console.log('LSThemeController::toggle() new isDark=' + this.#isDark);
    }

    getCurrent() {
        return this.#palette;
    }

    isDark() {
        return this.#isDark;
    }
}

class LSToastController {

    #fadeTimeout = 500;
    #showTime = 5000;
    #toastId = null;
    #toast = null;
    #position = 'toast-bottom-right';
    #type = 'info-background';
    #withHeader = false;

    constructor(position = this.#position, type=this.#type, fadeTimeout = this.#fadeTimeout, showTime = this.#showTime) {
        this.#type = type;
        this.#fadeTimeout = fadeTimeout;
        this.#showTime = showTime;

        this.#toastId = `toast-${Date.now()}-${Math.floor(Math.random() * 1000)}`;
        this.#toast = document.createElement('div');
        this.#toast.setAttribute('id', this.#toastId);
        this.#toast.classList.add('toast');
        this.#toast.classList.add(this.#position);
        document.body.appendChild(this.#toast);
        this.setPosition(position);
    }

    static init() {
        let state = true;
        if(state) {
            state = false;
        }
    }

    show(message, header = null) {
        if(header) {
            this.#toast.innerHTML = `
                <div class="toast-header ${this.#type}">
                    <div class="toast-label">${header}</div>
                    <button type="button" class="button close" data-parent="${this.#toastId}"></button>
                </div>
                <div class="toast-content text-color">${message}</div>
            `;
        } else {
            this.#toast.classList.add(this.#type);
            this.#toast.innerHTML = `
                <div class="toast-content ${this.#type}">${message}</div>
                <button type="button" class="button close mr-3" data-parent="${this.#toastId}"></button>
            `;
        }

        this.#toast.classList.add('show');

        this.#toast.querySelector('button.close').addEventListener('click', () => {
            this.#hide();
        });

        setTimeout(() => this.#hide(), this.#showTime);
    }

    setPosition(position) {
        this.#toast.classList.remove(this.#position);
        this.#position = position
        this.#toast.classList.add(this.#position);
    }

    #hide() {
        this.#toast.classList.add('fade-out');
        setTimeout(() => this.#toast.remove(), this.#fadeTimeout);
    }
}

class LSAnimations {

    static init() {}

    static fadeIn(element, duration = 1000) {
        element.style.opacity = 0;
        element.style.transition = `opacity ${duration}ms`;
        requestAnimationFrame(() => {
            element.style.opacity = 1;
        });
    }

    static fadeOut(element, duration = 1000) {
        element.style.opacity = 1;
        element.style.transition = `opacity ${duration}ms`;
        requestAnimationFrame(() => {
            element.style.opacity = 0;
        });
    }

    static slideIn(element, duration = 1000, direction = 'left') {
        element.style.transition = `transform ${duration}ms`;
        element.style.transform = `translateX(${direction === 'left' ? '-' : ''}100vh)`;
        requestAnimationFrame(() => {
            element.style.transform = 'translateX(0)';
        });
    }

    static slideOut(element, duration = 1000, direction = 'left') {
        element.style.transition = `transform ${duration}ms`;
        requestAnimationFrame(() => {
            element.style.transform = `translateX(${direction === 'left' ? '-' : ''}100vh)`;
        });
    }

    static scale(element, scaleValue = 1, duration = 1000) {
        element.style.transition = `transform ${duration}ms`;
        requestAnimationFrame(() => {
            element.style.transform = `scale(${scaleValue})`;
        });
    }

    static rotate(element, degrees = 0, duration = 1000) {
        element.style.transition = `transform ${duration}ms`;
        requestAnimationFrame(() => {
            element.style.transform = `rotate(${degrees}deg)`;
        });
    }

    static move(element, x = 0, y = 0, duration = 1000) {
        element.style.transition = `transform ${duration}ms`;
        requestAnimationFrame(() => {
            element.style.transform = `translate(${x}px, ${y}px)`;
        });
    }

    static bounce(element, duration = 1000, times = 3) {
        let count = 0;
        const bounce = () => {
            if (count >= times) return;
            element.style.transition = `transform ${duration / (times * 2)}ms`;
            element.style.transform = `translateY(-20px)`;
            setTimeout(() => {
                element.style.transform = 'translateY(0)';
                count++;
                setTimeout(bounce, (duration / (times * 2)));
            }, (duration / (times * 2)));
        }
        bounce();
    }

    static shake(element, durationInSeconds = 0.5) {
        element.style.transition = `transform ${durationInSeconds / 10}s`; 
        let direction = 1;
        let shakeCount = 0;
        const shakeInterval = setInterval(() => {
            element.style.transform = `translateX(${direction * 5}px)`;
            direction *= -1;
            shakeCount++;
            if (shakeCount > 10) {
                clearInterval(shakeInterval);
                element.style.transform = 'translateX(0)';
            }
        }, (durationInSeconds / 10) * 1000);
    }
}

class LSAnimationController {
    #elements = [];
    #observer;
    #options;
    #defaultSelectors = {
        fadeIn: '.fade-in',
        fadeInTop: '.fade-in-top',
        fadeInBottom: '.fade-in-bottom',
        fadeInLeft: '.fade-in-left',
        fadeInRight: '.fade-in-right',
        fadeOut: '.fade-out',
        slideLeft: '.slide-in-left',
        slideRight: '.slide-in-right',
        slideBottom: '.slide-in-bottom',
        slideTop: '.slide-in-top',
    };

    static init() {}

    constructor(options = { threshold: 0.1, delay: 100 }, selectors = null) {
        this.#options = options;
        this.selectors = selectors && Object.keys(selectors).length ? selectors : this.#defaultSelectors;

        Object.values(this.selectors).forEach(selector => {
            const elements = document.querySelectorAll(selector);
            elements.forEach(element => {
                this.#elements.push(element);
            });
        });

        this.#elements.sort((a, b) => {
            const orderA = parseInt(a.getAttribute('data-anim-order')) || 0;
            const orderB = parseInt(b.getAttribute('data-anim-order')) || 0;
            console.log(`orderA: ${orderA} | orderB: ${orderB}`);
            return orderA - orderB;
        });

        this.#elements.forEach((element, index) => this.#createObserver(element, index));
    }

    #createObserver(element, index) {
        const observer = new IntersectionObserver((entries) => {
            entries.forEach(entry => {
                if (entry.isIntersecting) {
                    setTimeout(() => {
                        entry.target.classList.add('visible');
                        observer.unobserve(entry.target);
                    }, index * this.#options.delay);
                }
            });
        }, this.#options);

        observer.observe(element);
    }
}

class LSSmoothScroll {

    static init() {}

    constructor(duration = 800) {
        this.duration = duration;
        this.isScrolling = false;
        this.handleWheelEvent = this.handleWheelEvent.bind(this);

        window.addEventListener('wheel', this.handleWheelEvent, { passive: false });
    }

    handleWheelEvent(event) {
        event.preventDefault();
        if(this.isScrolling) return;

        this.smoothScroll(event.deltaY);
    }

    smoothScroll(scrollDistance) {
        this.isScrolling = true;

        const startPosition = window.scrollY;
        const targetPosition = startPosition + scrollDistance;
        let start = null;

        const animateScroll = (time) => {
            if(!start) start = time;
            const progress = time - start;
            const position = this.easeInOutQuad(progress, startPosition, scrollDistance, this.duration);
            window.scrollTo(0, position);

            if(progress < this.duration) {
                requestAnimationFrame(animateScroll);
            } else {
                this.isScrolling = false;
            }
        };

        requestAnimationFrame(animateScroll);
    }

    easeInOutQuad(t, b, c, d) {
        t /= d / 2;
        if(t < 1) return c / 2 * t * t + b;
        t--;
        return -c / 2 * (t * (t - 2) - 1) + b;
    }
}

class LSSideMenuController {
	#menu;
	#wrapper;
	#toggler;
	#pinner;
	#isMenuOpen = false;
	#isMenuPinned = false;
	#isMenuToggleable = true;
	#menuState = "open";

	constructor({
		menuSelector = '[data-role="sidemenu"]',
		wrapperSelector = '[data-role="site-wrapper"]',
		togglerSelector = '[data-role="sidemenu-toggler"]',
		pinnerSelector = '[data-role="sidemenu-pinner"]',
		initialState = "open",
	} = {}) {
		this.#menu = document.querySelector(menuSelector);
		this.#wrapper = document.querySelector(wrapperSelector);
		this.#toggler = document.querySelector(togglerSelector);
		this.#pinner = document.querySelector(pinnerSelector);
		this.#menuState = initialState;

		this.#initializeMenu();
	}

	#initializeMenu() {
		this.#pinner.addEventListener("click", () => this.#pinMenu());
		this.#toggler.addEventListener("click", () => this.#toggleMenu());
		this.#setInitialState();
		this.#configureHoverEvents();
	}

	#setInitialState() {
		if (this.#menuState === "open" || this.#menuState === "compact") {
			this.#menu.classList.add(this.#menuState);
		}

		if (this.#menu.classList.contains("open")) {
			this.#wrapper.classList.add("with-sidemenu");
			this.#isMenuOpen = true;
			this.#isMenuPinned = true;
		} else if (this.#menu.classList.contains("compact")) {
			this.#wrapper.classList.remove("with-sidemenu");
			this.#isMenuOpen = false;
			this.#isMenuPinned = false;
		}
	}

	#setMenuVisibility() {
		if (this.#isMenuToggleable) {
			if (this.#isMenuOpen) {
				this.#menu.classList.remove("open");
				this.#wrapper.classList.remove("with-sidemenu");
				this.#menu.classList.add("compact");
			} else {
				this.#menu.classList.remove("compact");
				this.#menu.classList.add("open");
				this.#wrapper.classList.add("with-sidemenu");
			}
			this.#isMenuOpen = !this.#isMenuOpen;
		} else {
			this.#menu.classList.remove("open");
			this.#wrapper.classList.remove("with-sidemenu");
			this.#menu.classList.add("compact");
			this.#isMenuOpen = false;
		}
	}

	#configureHoverEvents() {
		const icon = this.#menu.querySelector(
			'[data-role="sidemenu-pinner"] i'
		);
		if (this.#isMenuPinned || !this.#isMenuToggleable) {
			this.#menu.removeEventListener("mouseenter", () => this.#setMenuVisibility());
			this.#menu.removeEventListener("mouseleave", () => this.#setMenuVisibility());
			icon.classList.replace("fa-thumbtack", "fa-times-circle");
		} else {
			this.#menu.addEventListener("mouseenter", () => this.#setMenuVisibility());
			this.#menu.addEventListener("mouseleave", () => this.#setMenuVisibility());
			icon.classList.replace("fa-times-circle", "fa-thumbtack");
		}
	}

	#pinMenu() {
		this.#isMenuPinned = !this.#isMenuPinned;
		this.#configureHoverEvents();
	}

	#toggleMenu() {
		const icon = this.#toggler.querySelector("i");
		this.#isMenuToggleable = !this.#isMenuToggleable;
		this.#isMenuToggleable
			? icon.classList.replace("keep-compact", "keep-open")
			: icon.classList.replace("keep-open", "keep-compact");
		this.#setMenuVisibility();
	}
}


